/***************************************************************************
                          ToyMem.c  -  Core code for the Toy Memory Manager.
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca
***************************************************************************/

#include<malloc.h>
#include<string.h>
#include<stdio.h>

#include "ToySegment.h"
#include "ToySegmentList.h"
#include "ToyProcess.h"
#include "ToyMemType.h"
#include "ToyMem.h"
#include "ToyMem_Internal.h"
#include "ToyLoad.h"


/**
 *  ERR_UNKNOWN and ERR_NONE do not have strings in the table.
 */
const char *ErrMessages[] =
{
  "Invalid syntax in processor input"
, "Process already loaded"
, "Cannot open load file"
, "Load file has invalid format"
, "Excess values in load file"
, "Invalid instruction in load file"
, "Invalid data value in load file"
, "Insufficient main memory to load file"
, "Address is outside of valid segments"
, "Specified process does not exist"
, "Insufficient privilege for PrintAllMemory"
, "END command reached, parser stopped"
, "Command not yet implemented"
, "Insufficient heap space"
, "Invalid address for FreeHeap"
, "Invalid address for ReallocateHeap"
, NULL                                         };

/**
 * Always returns ERR_NONE.
 */
ErrMessage_t TMM_MarkMemEmpty(ToyMM *tmm,int addr, int len)
{
  int n;
  for(n=addr; n<(addr+len); n++)
  {
    tmm->mem[n].type=DATA_ERROR;
  }
  return(ERR_NONE);
}

/**
 * Always returns ERR_NONE.
 */
ErrMessage_t TMM_CopyMem(ToyMM *tmm, int destaddr, int srcaddr, int len)
{
  if(tmm==NULL)  return(ERR_UNKNOWN);

  while(len>0)
  {
    tmm->mem[destaddr++]=tmm->mem[srcaddr++];
    len--;
  }
  return(ERR_NONE);
}

/**
 *  Called directly in response to the "PrintAllMemory" command.
 */

ErrMessage_t TMM_PrintAllMemory(ToyMM *tmm)
{
  int n;
  SegList *list=NULL;
  SegList *curSeg=NULL;

  if(tmm==NULL) return(ERR_UNKNOWN);

  /*    Get list of all segments  */
  for(n=0; n<MAX_PROCESSES; n++)
  {
    SegList *copy=NULL;
    if(tmm->processList[n]==NULL) continue;

    copy=TMM_CopySegList(tmm->processList[n]->segments);
    copy=TMM_SeekSegListHead(copy);
    list=TMM_AppendSegList(list,copy);
    list=TMM_SeekSegListHead(list);
  }
  list=TMM_SortSegList(list);
  list=TMM_SeekSegListHead(list);

  printf(LINE_RULE);
  printf(LINE_PREFIX"Memory Contents:\n");

  curSeg=list;
  while(curSeg!=NULL)
  {
    int n;
    for(n=curSeg->seg.pos; n<(curSeg->seg.pos+curSeg->seg.len); n++)
    {
      TMM_PrintMem(tmm,n);
    }
    curSeg=curSeg->next;
  }
  TMM_FreeSegList(list);
  printf(LINE_RULE);

  return(ERR_NONE);
}

/**
 *  Called directly in response to the "PrintProcessTable"
 *  command.
 */
ErrMessage_t TMM_PrintProcessTable(ToyMM *tm, int process)
{
  int n;

  if((process<0)||(process>=MAX_PROCESSES))              return(ERR_UNKNOWN);
  else if((process!=0)&&(tm->processList[process]==NULL))return(ERR_NOPROCESS);

  printf(LINE_RULE);
  printf(LINE_PREFIX"Process Table:\n");
  printf(LINE_PREFIX" PID #SEG SEG1 LEN1 SEG2 LEN2 SEG3 LEN3 SEG4 LEN4\n");
  printf(LINE_PREFIX" --- ---- ---- ---- ---- ---- ---- ---- ---- ----\n");

  for(n=0; n<MAX_PROCESSES; n++)
  {
    int i=-1;
    int numSegs     = -1;
    int segSizes[4] = {-1,-1,-1,-1};
    int segPos[4]   = {-1,-1,-1,-1};
    SegList *list=NULL;

    if(tm->processList[n]==NULL)  continue;
    list=TMM_SeekSegListHead(tm->processList[n]->segments);

    for(numSegs=0; (list!=NULL)&&(numSegs<4); list=list->next,numSegs++ )
    {
      segSizes[numSegs]=list->seg.len;
      segPos[numSegs]=list->seg.pos;
    }
    printf(LINE_PREFIX"  P%1d    %1d",n,numSegs);
    for(i=0; i<numSegs; i++)
    {
      printf(" %4d",segPos[i]);
      printf(" %4d",segSizes[i]);
    }
    printf("\n");
  }

  printf(LINE_RULE);

  return(ERR_NONE);
}

/**
 *  Called directly in response to a "PrintProcessMemory" command.
 */
ErrMessage_t TMM_PrintProcessMemory(ToyMM *tm, int process)
{
  int n=-1;
  int seg=0;
  SegList *list=NULL;

  if( (process<0)||(process>=MAX_PROCESSES))  return(ERR_UNKNOWN);
  if(tm->processList[process]==NULL)          return(ERR_NOPROCESS);

  printf(LINE_RULE);
  printf(LINE_PREFIX"Memory Contents:\n");

  list=tm->processList[process]->segments;
  list=TMM_SeekSegListHead(list);

  while(list!=NULL) /*  Loop through every segment. */
  {
    if(seg==SEG_HEAP)
    {
      int hpos=0;
      SegList *heap=tm->processList[process]->usedheap;
      while(heap!=NULL)
      {
        /* Loop through every address in used heap segs */
        for(hpos=heap->seg.pos; hpos<(heap->seg.pos+heap->seg.len); hpos++)
        {
          TMM_PrintMem(tm,hpos);          
        }
        heap=heap->next;
      }
    }
    else
    {
      /*  Loop through every address in segment.  */
      for(n=list->seg.pos; n<(list->seg.pos+list->seg.len); n++)
      {
        /*  Attempt to print every valid memory area.
            Invalid ones will not be printed. */
        TMM_PrintMem(tm,n);
      }
    }
    
    list=list->next;
    seg++;
  }

  printf(LINE_RULE);
  return(ERR_NONE);
}

/**
 *  One of two variations on STORE;  this one attempts to store
 *  an integer in the specified address in process memory.
 */
ErrMessage_t TMM_StoreInteger(ToyMM *tm,int process,int addr,int val)
{
  SegList *list;
  int seg=SEG_CODE;
  if(tm->processList[process]==NULL)  return(ERR_NOPROCESS);

  list=tm->processList[process]->segments;
  list=TMM_SeekSegListHead(list);

  list=list->next;  /*  Segment 0 is code segment, can't put data in it */
  seg++;

  while(list!=NULL) /*  Loop through segments */
  {
    /*  Test if address resides in segment  */
    if( (addr>=list->seg.pos) && (addr< (list->seg.pos+list->seg.len)) )
    {
      if(seg==SEG_HEAP)
      {
        return(TMM_StoreHeapInteger(tm,process,addr,val));
      }

      /*  Set the memory type and value */
      tm->mem[addr].type=DATA_INTEGER;
      tm->mem[addr].integer.value=val;
      return(ERR_NONE); /*  Return success  */
    }
    list=list->next;
    seg++;
  }

  /*  If we didn't find it, return error. */
  return(ERR_INVALIDADDRESS);
}

/**
 *  The second variation on STORE;  attempts to store a string.
 */
ErrMessage_t TMM_StoreString(ToyMM *tm, int process, int addr, const char *str)
{
  SegList *list;
  int seg=SEG_CODE;

  if(strlen(str)>MAX_STRLEN) return(ERR_INVALIDINPUT);
  if(tm->processList[process]==NULL)  return(ERR_NOPROCESS);

  list=tm->processList[process]->segments;
  list=TMM_SeekSegListHead(list);

  list=list->next;  /*  Segment 0 is code segment, can't put data in it */
  seg++;

  while(list!=NULL) /*  Loop through segments */
  {
    /*  Test if address resides in segment  */
    if( (addr>=list->seg.pos) && (addr< (list->seg.pos+list->seg.len)) )
    {
      if(seg==SEG_HEAP)
      {
        return(TMM_StoreHeapString(tm,process,addr,str));
      }

      /*  Set the memory type and value */
      tm->mem[addr].type=DATA_STRING;
      strcpy(tm->mem[addr].string.value,str);
      return(ERR_NONE); /*  Return success  */
    }
    list=list->next;
    seg++;
  }

  /*  If we didn't find it, return error. */
  return(ERR_INVALIDADDRESS);
}

/******************************************************************************/

/**
 * Returns 0 on success, -1, on failure.
 * \sa TMM_AllocateProcessAt()
 */
ErrMessage_t TMM_TerminateProcess(ToyMM *tmm, int ID)
{
  SegList *list;

  /*  Can't free a nonexistent process  */
  if(tmm->processList[ID]==NULL)  return(ERR_NOPROCESS);

  /*  Get the list and go to it's start */
  list=tmm->processList[ID]->segments;
  list=TMM_SeekSegListHead(list);

  /*  Loop through entire list  */
  while(list!=NULL)
  { /*  Free every segment in process */
    TMM_FreeSegment(tmm,list->seg.pos,list->seg.len);
    list=list->next;
  }

  /*  Free the segment list itself from REAL memory */
  if(tmm->processList[ID]->segments!=NULL)
    TMM_FreeSegList(tmm->processList[ID]->segments);

  if(tmm->processList[ID]->freeheap!=NULL)
    TMM_FreeSegList(tmm->processList[ID]->freeheap);

  if(tmm->processList[ID]->usedheap!=NULL)
    TMM_FreeSegList(tmm->processList[ID]->usedheap);

  /*  Free the process itself */
  free(tmm->processList[ID]);

  /*  Mark the process slot as empty  */
  tmm->processList[ID]=NULL;

  return(ERR_NONE);
}

/**
 * \sa TMM_TerminateProcess()
 */
ErrMessage_t TMM_AllocateProcessAt(ToyMM *tmm, int ID, int pos, int nSegs, int size[])
{
  int n=-1,startpos=-1;
  Process *p=NULL;
  SegList *list=NULL;

  if((ID>=MAX_PROCESSES)||(ID<0)) return(ERR_UNKNOWN);
  if(tmm->processList[ID]!=NULL)  return(ERR_NOPROCESS);

  p=(Process *)malloc(sizeof(Process));
  if(p==NULL) return(ERR_UNKNOWN);

  p->ptr=NULL;
  p->freeheap=NULL;
  p->usedheap=NULL;
  p->segments=NULL;
  p->ID=ID;

  /*  Add the process in the memory manager */
  tmm->processList[ID]=p;

  /*  check for overlapping memory areas  */
  for(n=0,startpos=pos; n<nSegs; n++)
  {
    SegList *node=NULL;

    /*  Try and create a new segment at specified pos */
    if(TMM_AllocateSegmentAt(tmm,startpos,size[n])!=ERR_NONE)
    {
      /*  Free p and all it's segments  */
      TMM_TerminateProcess(tmm,ID);
      return(ERR_INSUFFICIENTMEM);
    }

    /*  Create a node representing the new segment  */
    node=TMM_CreateSegList(startpos,size[n],NULL,NULL);

    /*  Add it to the list of valid segs in p */
    p->segments=TMM_AppendSegList(p->segments,node);

    /*  Increment starting position for next segment */
    startpos+=size[n];
  }

  /*  Seek to the head of the list  */
  p->segments=TMM_SeekSegListHead(p->segments);

  list=TMM_GetSegmentNode(tmm,ID,SEG_HEAP);
  if(list!=NULL)  /* If a heap segment exists for this process */
  {    /* This node represents free memory in the heap segment */
    SegList *node=TMM_CreateSegList(list->seg.pos,list->seg.len,NULL,NULL);
    p->freeheap=node;
    p->ptr=p->freeheap;
  }

  return(ERR_NONE);    /*  Yay!  It worked!  */
}

/*
 *  Allocates a process, along with the physical and virtual memory
 *  associated with it.  Returns 0 on success, -1 on failure.
 *
 *  Note - Depreciated.  Update or throw out.
 */
int TMM_AllocateProcess(ToyMM *tmm,int ID, int nSegs, int size[])
{
  int n=-1;

  SegList *list=NULL;
  Process *p=NULL;

  if((ID>=MAX_PROCESSES)||(ID<0)) return(-1);
  if(tmm->processList[ID]!=NULL)  return(-1);

  p=(Process *)malloc(sizeof(Process));

  if(p==NULL) return(ERR_UNKNOWN); /*  Out of memory */

  memset(p,0,sizeof(Process));

  p->ID=ID;

  for(n=0; n<nSegs; n++)
  {
    int v=TMM_AllocateSegment(tmm,size[n]);
    if(v>=0)
    {
      list=TMM_AppendSegList(list,TMM_CreateSegList(v,size[n],NULL,NULL));
    }
    else
    {

      /* Out of virtual memory */
      SegList *lst2=list;

      while(lst2!=NULL)
      {
        TMM_FreeSegment(tmm,lst2->seg.pos,lst2->seg.len);
        lst2=lst2->next;

      }

      TMM_FreeSegList(list);
      return(ERR_INSUFFICIENTMEM);
    }
  }

  p->segments=TMM_SeekSegListHead(list);
  tmm->processList[ID]=p;
  return(ERR_NONE);
}

/**
 *  \sa TMM_FreeSegment()
 */
ErrMessage_t TMM_AllocateSegmentAt(ToyMM *tmm, int pos, int size)
{
  int i=-1,overlap=0;
  ToySegment mySeg;
  mySeg.pos=pos;
  mySeg.len=size;

  /*  Test for out-of-range mem */
  if((pos+size)>tmm->memSize)  return(ERR_INSUFFICIENTMEM);

  /*  Test for overlapping segments */
  for(i=0,overlap=0; (i<MAX_PROCESSES)&&(overlap==0); i++)

  {
    SegList *list=NULL;
    /*  Don't check nonexistent processes */
    if(tmm->processList[i]!=NULL) list=tmm->processList[i]->segments;
    while(list!=NULL)
    {
      overlap|=TMM_AreaIntersectsSeg(&mySeg,&(list->seg));
      list=list->next;
    }
  }

  if(overlap!=0)
  {
    return(ERR_INSUFFICIENTMEM); /*  memory range not free */
  }
  else
  {
    /*  Clear the memory so we can see what's been used */
    for(i=pos; i<(pos+size); i++) tmm->mem[i].type=DATA_ERROR;
    tmm->memFree=TMM_RemoveAreaFromSegList(tmm->memFree,pos,size);
    return(ERR_NONE);
  }
}

/*
 *  Allocates a segment of virtual memory, if possible.
 *  Returns the starting address of the segment on success, or
 *  -1 on failure.
 *  note - depreciated, update or throw out
 */
int TMM_AllocateSegment(ToyMM *tmm, int size)
{
  int pos=TMM_FindSegSpace(tmm->memFree,size);
  if(pos<0) return(-1);
  else
  {
    int i;


    /*  Clear the memory so we can see what's been used */
    for(i=pos; i<(pos+size); i++) tmm->mem[i].type=DATA_ERROR;
    tmm->memFree=TMM_RemoveAreaFromSegList(tmm->memFree,pos,size);
    TMM_SortSegList(tmm->memFree);
    return(pos);
  }
}

/**
 *  Also adds the segment to the Toy Memory manager's free list.
 */
ErrMessage_t TMM_FreeSegment(ToyMM *tmm, int pos, int size)
{
  int n;
  SegList *list;

  for(n=0; n<MAX_PROCESSES; n++)
  {
    if(tmm->processList[n]==NULL)	continue;

    list=tmm->processList[n]->segments;
    while(list!=NULL)
    {
      if((list->seg.len==size)&&(list->seg.pos==pos))
      {
        list=TMM_AddAreaToSegList(tmm->memFree,pos,size);
        if(list==NULL)
        {
          /* Error in deallocation */
          return(ERR_UNKNOWN);
        }
        else
        {

          tmm->memFree=list;
          return(ERR_NONE);
        }
      }

      list=list->next;
    }
  }
  /* Couldn't find it. */
  return(ERR_UNKNOWN);
}


/**
 *  \sa TMM_Free()
 */
ToyMM *TMM_Create(int memSize)
{
  int n;

  ToyMM *tmm = (ToyMM *)malloc(sizeof(ToyMM));

  if(tmm==NULL)	/*  Out of memory */
  {
    return(NULL);
  }
  else
  {
    for(n=0; n<MAX_PROCESSES; n++)
    {
      tmm->processList[n]=NULL;
    }

    tmm->mem=(ToyMem *)malloc(memSize*sizeof(ToyMem));
    memset(tmm->mem,0,memSize*sizeof(ToyMem));

    tmm->memFree = TMM_CreateSegList(0,memSize,NULL,NULL);

    tmm->memSize = memSize;

    return(tmm);
  }
}

/**
 *  \sa TMM_Create()
 */
ErrMessage_t TMM_Free(ToyMM *tmm)
{
  int n;

  if(tmm==NULL) return(ERR_UNKNOWN);

  for(n=0; n<MAX_PROCESSES; n++)
  {
    if(tmm->processList[n])	TMM_TerminateProcess(tmm,n);
  }

  free(tmm->mem);
  TMM_FreeSegList(tmm->memFree);

  return(ERR_NONE);
}

/**
 *  Attempts to print one memory element
 *  from an absolute memory address in tmm.
 *  Does not print anything if the memory is
 *  empty or invalid.
 */
void TMM_PrintMem(ToyMM *tmm, int addr)
{
  ToyMem *mem=tmm->mem+addr;


  if(mem->type==DATA_ERROR) return;

  printf(LINE_PREFIX" %3d ",addr);
  switch(mem->type)
  {
  case DATA_ERROR:
    break;

  case DATA_INTEGER:
    printf("%d\n",mem->integer.value);
    break;
  case DATA_INSTRUCTION:
    printf("%s\n",DataType_List[DATA_INSTRUCTION][mem->instruction.value]);
    break;
  case DATA_STRING:
    printf("%s\n",mem->string.value);
    break;

  default:

    printf("Unknown mem type %d, ",mem->type);
    printf("Email me at tsm@accesscomm.ca and tell me how you did this.\n");
    break;
  }
}

/*! \brief  Returns the requested segment from a process in tm. */
/**
 *  Returns NULL if nonexistent.
 */
SegList *TMM_GetSegmentNode(ToyMM *tm, int process, SegType_e sg)
{
  SegList *list;
  int seg=sg;

  if((process<0)||(process>=MAX_PROCESSES)) return(NULL);
  if(seg<0)                                 return(NULL);
  if(tm->processList[process]==NULL)        return(NULL);

  list=TMM_SeekSegListHead(tm->processList[process]->segments);

  while((list!=NULL)&&(seg>0))
  {
    seg--;
    list=list->next;
  }

  return(list);
}

/*! \brief  Returns the starting address of the requested segment of a process in tm.  Returns -1 if nonexistent.*/
int TMM_GetSegmentAddr(ToyMM *tm, int process, SegType_e seg)
{
  SegList *list=TMM_GetSegmentNode(tm,process,seg);

  if(list!=NULL)  return(list->seg.pos);
  else            return(-1);
}

