/***************************************************************************
                          ToyMem.h  -  description
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

#ifndef __TOYMEM_H__
#define __TOYMEM_H__

#ifdef __cplusplus
  extern "C" {
#endif/*__cplusplus*/

/*! \brief Maximum number of processes       */
#define MAX_PROCESSES 10
/*! \brief Minimum allowable number of memory elements */
#define MIN_MEMSIZE   1000  

/*! \brief Structure that defines a Toy Memory Manager. */
/**
 *  Contains an array of pointers to processes(NULL denotes no process),
 *  an array of memory elements, a linked list of free space,
 *  and a member describing the size of the memory array.
 */
typedef struct ToyMM
{
  /*! \brief Array of processes */
  struct Process *processList[MAX_PROCESSES];
  /*! \brief Array of memory elements */
  union ToyMem   *mem;
  /*! \brief Size of memory element array */
  int             memSize;
  /*! \brief Linked list of free space holes */
  struct SegList *memFree;

} ToyMM;

/**
 * Defines to simplify text output
 */
/*! \brief Error message template  */ 
#define ERR_PREFIX  "******* ERROR: "
/*! \brief Line rule  */
#define LINE_RULE   "+--------------------------------------------------\n"
/*! \brief Normal line prefix */
#define LINE_PREFIX "| "
/*! \brief Format for printing line #s  */
#define LINE_NUMBER_FORMAT "%4d"
/*! \brief Simple macro to print an error message to stdout */
#define PRINT_ERROR(error) printf(ERR_PREFIX"%s\n",ErrMessages[error])

/*! \brief Enumerated type listing valid error return values. */
/**
 *  All values except ERR_UNKNOWN and ERR_NONE have accompanying
 *  values in the string table ErrMessages[], for example
 *  ErrMessages[ERR_INVALIDINPUT]
 */
typedef enum ErrMessage_t
{
  /*! \brief Unknown error	*/
  ERR_UNKNOWN=-1,
  /*! \brief Bad command file syntax. */
  ERR_INVALIDINPUT=0,
  /*! \brief CreateProcess attempted to replace an existing process. */
  ERR_PROCESSEXISTS=1,
  /*! \brief .lod file does not exist. */
  ERR_NOFILE,
  /*! \brief Bad .lod file format. */
  ERR_INVALIDFILE,
  /*! \brief .lod file contains too many values for segments. */
  ERR_DATAOVERFLOW,
  /*! \brief .lod file contains bad instructions. */
  ERR_INVALIDINSTRUCTION,
  /*! \brief .lod file contains invalid values. */
  ERR_INVALIDDATA,
  /*! \brief Not enough memory to create process.*/
  ERR_INSUFFICIENTMEM,
  /*! \brief Address outside of valid segments. */
  ERR_INVALIDADDRESS,
  /*! \brief Operation attempted on nonexistent process. */
  ERR_NOPROCESS,
  /*! \brief Privileged operation attempted by unpriveleged process. */
  ERR_NOPRIVILEGE,
  /*! \brief END reached, parsing stopped. */
  ERR_PARSEEND,
  /*! \brief Function not yet implemented. */
  ERR_UNIMPLEMENTED,
  /*! \brief No space in heap to allocate data. */
  ERR_HEAPFULL,
  /*! \brief Heap variable not allocated at this address. */
  ERR_BADFREEHEAPADDRESS,
  /*! \brief Bad address for ReallocateHeap. */
  ERR_BADREALLOCADDRESS,
  /*! \brief No error. */
  ERR_NONE
} ErrMessage_t;

/*! \brief String table of error message strings to match ErrMessage_t */
extern const char *ErrMessages[];

/*! \brief Creates a new Toy Memory Manager */
ToyMM *TMM_Create(int memsize);

/*! \brief Frees a Toy Memory Manager */
ErrMessage_t TMM_Free(ToyMM *tmm);

/*! \brief Function to parse a commandfile */
int TMM_ParseFile(const char *fname, ToyMM *tm);

#ifdef __cplusplus
  }
#endif/*__cplusplus*/
 
#endif/*__TOYMEM_H__*/
