/***************************************************************************
                          ToyMemType.h  -  Defines the structures and
    main union that represent the Toy Memory Manager's main memory.
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

#ifndef __TOYMEMTYPE_H__
#define __TOYMEMTYPE_H__

#ifdef __cplusplus
  extern "C" {
#endif/*__cplusplus*/

/*! \brief Maximum length for string memory items */
#define MAX_STRLEN 20

/*
 *  Toy_MemType.h
 *  Defines the structures used for the type of memory
 *  the Toy Memory Manager handles;  a single slot in memory
 *  can be an integer, a string, or an instruction.
 *
 */

/*! \brief Enumerates the valid instruction types. */
typedef enum InstructionType_e
{
  /*! \brief Invalid instruction. */
  INSTRUCTION_ERROR = -1,
  /*! \brief Run. */  
  INSTRUCTION_RUN   = 0,
  /*! \brief Halt. */  
  INSTRUCTION_HALT
} InstructionType_e;

/*! \brief Enumerated type denoting valid datatypes for a single memory unit. */
typedef enum DataType_e
{
  /*! \brief Invalid data value. */
  DATA_ERROR    =-1,
  /*! \brief The 'magic' sequence denoting a .lod file header */  
  DATA_MAGIC    = 0,
  /*! \brief Denoes an integer*/  
  DATA_INTEGER  = 1,
  /*! \brief Denotes a string*/
  DATA_STRING,
  /*! \brief Denotes an instruction */  
  DATA_INSTRUCTION
} DataType_e;

/*! \brief Structure to hold a single integer. */
typedef struct ToyMem_Integer
{
  /*! \brief Type information - DATA_INTEGER */
  DataType_e type;
  /*! \brief Integer value */  
  unsigned int value;
} ToyMem_Integer;

/*! \brief Structure to hold a NULL-terminated ASCII string up to MAX_STRLEN chars in length. */
typedef struct ToyMem_String
{
  /*! \brief Type information - DATA_STRING */
  DataType_e type;
  /*! \brief The string itself */  
  char value[MAX_STRLEN+1];
} ToyMem_String;

/*! \brief Holds one instruction. */
typedef struct ToyMem_Instruction
{
  /*! \brief Type information - DATA_INSTRUCTION*/  
  DataType_e type;
  /*! \brief The instruction itself */  
  InstructionType_e value;
} ToyMem_Instruction;

/*! \brief The main memory type union. */
/**
 *  Can hold strings, integers, or instructions.
 *  'type' is valid for every structure, so
 *  it is safe to be read from the union.
 *  DATA_ERROR denotes an unused memory
 *  item.
 *
 *  Note that all members occupy the same
 *  area in memory, so that only one of the
 *  instruction, string, and integer members
 *  can be valid at one time, as determined
 *  by type.
 */
typedef union ToyMem
{
  /*! \brief Type information */  
  DataType_e type;
  /*! \brief Instruction member */
  ToyMem_Instruction  instruction;
  /*! \brief String member */
  ToyMem_String       string;
  /*! \brief Integer member */  
  ToyMem_Integer      integer;  
} ToyMem;

#ifdef __cplusplus
  }
#endif/*__cplusplus*/

#endif/*__TOYMEMTYPE_H__*/
