/***************************************************************************
                          ToyMem_Internal.h  - Contains function headers
    that only ToyMem.c use.
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/
#ifndef __TOYMEM_INTERNAL_H__
#define __TOYMEM_INTERNAL_H__

#ifdef __cplusplus
  extern "C" {
#endif/*__cplusplus*/

/* Gets the address of a memory segment. */
int TMM_GetSegmentAddr(ToyMM *tm, int process, SegType_e seg);
/* Gets the linked-list node for a segment. */
struct SegList *TMM_GetSegmentNode(ToyMM *tm, int process, SegType_e seg);

/*! \brief Attempts to store an integer in the process' heap */
ErrMessage_t TMM_StoreHeapInteger(ToyMM *tm, int process, int addr, int value);

/*! \brief  Attempts to store a string in the process' heap */
ErrMessage_t TMM_StoreHeapString(ToyMM *tm, int process, int addr, const char *str);

/*! \brief Prints process table - running processes w/segment sizes, positions*/
ErrMessage_t TMM_PrintProcessTable(ToyMM *tm, int process);
/*! \brief Prints all memory contained in a process */
ErrMessage_t TMM_PrintProcessMemory(ToyMM *tm, int process);
/*! \brief Stores an integer in a process' memory */
ErrMessage_t TMM_StoreInteger(ToyMM *tm,int process,int addr,int val);
/*! \brief Stores a string in a process' memory */
ErrMessage_t TMM_StoreString(ToyMM *tm, int process, int addr, const char *str);
/*! \brief Terminates a process */
ErrMessage_t TMM_TerminateProcess(ToyMM *tm, int process);
/*! \brief Prints ALL used/modified memory.  Only process 0 can do this. */
ErrMessage_t TMM_PrintAllMemory(ToyMM *tm);
/*! \brief Allocates a variable in the heap. */
ErrMessage_t TMM_AllocateHeap(ToyMM *tm, int process, int size);
/*! \brief Reallocates a variable in the heap. */
ErrMessage_t TMM_ReallocateHeap(ToyMM *tm, int process, int addr, int size);
/*! \brief Frees a variable in the heap. */
ErrMessage_t TMM_FreeHeap(ToyMM *tm, int process, int addr);
/*! \brief Prints used portions of the heap. */
ErrMessage_t TMM_PrintHeap(ToyMM *tm, int process);


/*! \brief Attempts to allocate a segment at the specified address.   */
ErrMessage_t TMM_AllocateSegmentAt(ToyMM *tmm, int pos, int size);

/*! \brief Attempts to allocate a process at the specified address.  */
ErrMessage_t TMM_AllocateProcessAt(ToyMM *tm,int pID, int addr,int segs, int sizes[]);

/* ErrMessage_t TMM_AllocateProcess(ToyMM *tmm,int ID, int nSegs, int
size[]); */

/*! \brief Attempts to allocate a segment.  Returns address on success, -1 on failure. */
int TMM_AllocateSegment(ToyMM *tmm, int size);

/*! \brief  Attempts to free a segment. */
ErrMessage_t TMM_FreeSegment(ToyMM *tmm, int pos, int size);

/*! \brief  Unconditionally copy memory */
ErrMessage_t TMM_CopyMem(ToyMM *tmm, int destaddr, int srcaddr, int len);
/*! \brief Unconditionally mark memory as unused */
ErrMessage_t TMM_MarkMemEmpty(ToyMM *tmm,int addr, int len);

/*! \brief Prints the specified area of memory, if used  */
void TMM_PrintMem(ToyMM *tmm, int addr);

#ifdef __cplusplus
    }
#endif/*__cplusplus*/

#endif/*__TOYMEM_INTERNAL_H__*/

