/***************************************************************************
                          ToyParseCmd.h  -  Function definitions,
    enumerated types, and string tables used for parsing Command files.
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/
#ifndef __TOYPARSECMD_H__
#define __TOYPARSECMD_H__

#ifdef __cplusplus
  extern "C" {
#endif/*__cplusplus*/

/*! \brief Enumerated type to define valid type of address operators. */
typedef enum AddressType_e
{
  ADDRESS_ERROR=-1,
  /* \brief !@<value>, absolute address in process space */  
  ADDRESS_ABSOLUTE=0,
  /*! \brief  @H<value>, address relative to heap segment */  
  ADDRESS_HEAP,
  /*! \brief @D<value>, address relative to data segment*/  
  ADDRESS_DATA,
  /*! \brief @S<value>, address relative to stack segment */  
  ADDRESS_STACK,
  /*! \brief @C<value>, address relative to code segment*/  
  ADDRESS_CODE
} AddressType_e;

/*! \brief Enumerated type of valid commands for the commandfile parser. */
typedef enum CommandType_e
{
  /*! \brief Invalid command */
  COMMAND_ERROR    =-1,
  /*! \brief Loads a .lod file*/
  COMMAND_LOADFILE = 0,
  /*! \brief Prints the process table for a process */  
  COMMAND_PRINTPROCESSTABLE,
  /*! \brief Prints the memory for a process */  
  COMMAND_PRINTPROCESSMEMORY,
  /*! \brief Stores a variable in process memory */  
  COMMAND_STORE,
  /*! \brief Terminates a process */  
  COMMAND_TERMINATEPROCESS,
  /*! \brief Prints ALL memory.  Only P0 can do this */
  COMMAND_PRINTALLMEMORY,
  /*! \brief Stops parsing prematurely. */  
  COMMAND_END,
  /*! \brief Allocates a memory block in a process' heap segment*/  
  COMMAND_ALLOCATE,
  /*! \brief Reallocates a memory block in a process' heap segment */  
  COMMAND_REALLOCATE,
  /*! \brief Frees a memory block in a process' heap segment */  
  COMMAND_FREE,
  /*! \brief Prints the free and used lists for a process' heap*/  
  COMMAND_PRINTHEAP
} CommandType_e;

/*! \brief Enumereated type listing valid kinds of arguments. */
typedef enum ArgType_e
{
  /*! \brief Invalid argument. */  
  TYPE_ERROR    =-1,
  /*! \brief Process number */  
  TYPE_PROCESS  = 0,
  /*! \brief Command */  
  TYPE_COMMAND,
  /*! \brief Memory address */  
  TYPE_ADDRESS,
  /*! \brief Unsigned integer */  
  TYPE_INTEGER,
  /*! \brief ASCII string */  
  TYPE_STRING
} ArgType_e;

/*! \brief String table used by the commandfile parser */
extern const char **Arguments[];

#ifndef TMM_ParseFile
int TMM_ParseFile(const char *fname, ToyMM *tm);
#endif/*TMM_ParseFile*/

#ifdef __cplusplus
  }
#endif/*__cplusplus*/


#endif/*__TOYPARSECMD_H__*/
