/***************************************************************************
                          ToySegmentList.h
      A set of functions for creating, destroying, and
      manipulating doubly-linked lists of memory segments.

                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/
#ifndef __TOYSEGLIST_H__
#define __TOYSEGLIST_H__

#ifdef __cplusplus
  extern "C" {
#endif

/*! \brief Structure defining the nodes of linked lists of segments.*/
/**
 *  Doubly linked because they work without bother, and I'm lazy.
 */
typedef struct SegList
{
  /*! \brief Segment information */
  struct ToySegment seg;
  /*! \brief Previous node, or NULL if none  */
  struct SegList *prev;
  /*! \brief Next node, or NULL if none  */
  struct SegList *next;
} SegList;

/* ! \brief Convenience macro.  Returns the address of the last byte in the segment.*/
#define SegEnd(x) ((x)->pos + (x)->len)

/*! \brief Returns size of list in nodes */
int TMM_SegListSize(SegList *list);

/*! \brief Inserts a node into the list in sorted order. */
SegList *TMM_OrderedInsert(SegList *list, SegList *node);

/*! \brief Deletes a node from the list.  Does not deallocate. */
void TMM_RemoveFromList(SegList *node);

/*! \brief Makes a newly-allocated copy of a segment list */
SegList *TMM_CopySegList(SegList *list);

/*! \brief Creates a new node for a segment list. */
SegList *TMM_CreateSegList(int pos, int len, SegList *prev, SegList *next);

/*! \brief Returns the head of the passed list.*/
SegList *TMM_SeekSegListHead(SegList *list);
/*! \brief Returns the tail of the passed list.*/
SegList *TMM_SeekSegListEnd(SegList *list);

/*! \brief Appends a node or nodes to the end of the list. */
SegList *TMM_AppendSegList(SegList *list, SegList *node);

/*! \brief Frees a linked list of segments from memory */
int TMM_FreeSegList(SegList *list);

/*! \brief Compares two segments for sorting. */
int TMM_CompSegList(const SegList *l1, const SegList *l2);

/*! \brief Sorts the list in ascending order of segment address */
SegList *TMM_SortSegList(SegList *list);

/*! \brief Removes a memory area from the list */
SegList *TMM_RemoveAreaFromSegList(SegList *list, int pos, int len);

/*! \brief Merges end-to-end segments */
SegList *TMM_MergeSegList(SegList *list);


/*! \brief Returns nonzero if the segments overlap, 0 if not. */
int TMM_AreaIntersectsSeg(ToySegment *seg1, ToySegment *seg2);

/*! \brief Adds another segment to the list, if the new segment doesn't overlap with any existing ones. */
SegList *TMM_AddAreaToSegList(SegList *list, int pos, int len);

/*! \brief Prints the list of segments */
void TMM_PrintSegList(SegList *list);

/*! \brief Returns # of resultant segments. */
/**
 *  -1 means they don't intersect.
 *  0 means they cancelled out.
 *  1 or 2 means the result is in segOut.
 */
int TMM_SubtractSegs(ToySegment *seg1, ToySegment *seg2, ToySegment segOut[2]);

/*! \brief Tries to find space for a segment int a list of free memory segments. */
int TMM_FindSegSpace(SegList *list, int n);

#ifdef __cplusplus
  }
#endif

#endif/*__TOYSEGLIST_H__*/
